# -*- coding: utf-8 -*-
"""
app.fields
~~~~~~~~~~

This module defines extra WTForms fields

:copyright: (c) 2018 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import yaml
from wtforms import TextAreaField


class YAMLField(TextAreaField):
    """This field represents an HTML ``<textarea>`` used to input YAML"""

    def _value(self):
        return yaml.safe_dump(self.data, default_flow_style=False) if self.data else ""

    def process_formdata(self, valuelist):
        if valuelist:
            try:
                self.data = yaml.safe_load(valuelist[0])
            except yaml.YAMLError:
                raise ValueError("This field contains invalid YAML")
        else:
            self.data = None

    def pre_validate(self, form):
        super().pre_validate(form)
        if self.data:
            try:
                yaml.safe_dump(self.data)
            except yaml.YAMLError:
                raise ValueError("This field contains invalid YAML")
